use std::{error::Error, net::TcpListener};

use arti_client::{TorClient, TorClientConfig};
use futures::{Stream, StreamExt};
use log::info;
use tokio::io::split;
use tor_hsservice::config::OnionServiceConfigBuilder;
use tor_hsservice::handle_rend_requests;
use axum::{routing, Router};

use crate::tor_axum;


pub async fn start() -> Result<(), Box<dyn Error>> {
    let tor_client = TorClient::create_bootstrapped(TorClientConfig::default()).await?;

    let (onion_service, rend_requests) = tor_client.launch_onion_service(
        OnionServiceConfigBuilder::default()
            .nickname("gamal-service".to_owned().try_into().unwrap())
            .build()?,
    )?;
    
    let stream_requests = handle_rend_requests(rend_requests);
    
    let app = Router::new().route("/", routing::get(|| async { "Hello, World!" }));
    
    println!("serving at: http://{}", onion_service.onion_name().unwrap());
    
    tor_axum::serve(stream_requests, app).await;

    Ok(())
}